package cn.com.qmth.examcloud.core.questions.service.impl;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.print.api.CoursePaperCloudService;
import cn.com.qmth.examcloud.core.print.api.bean.SyncCoursePaperBean;
import cn.com.qmth.examcloud.core.print.api.request.SyncCoursePaperReq;
import cn.com.qmth.examcloud.core.print.api.response.SyncCoursePaperResp;
import cn.com.qmth.examcloud.core.questions.base.*;
import cn.com.qmth.examcloud.core.questions.base.enums.PaperStatus;
import cn.com.qmth.examcloud.core.questions.base.enums.PaperType;
import cn.com.qmth.examcloud.core.questions.base.exception.PaperException;
import cn.com.qmth.examcloud.core.questions.base.question.enums.QuesStructType;
import cn.com.qmth.examcloud.core.questions.base.word.DocxProcessUtil;
import cn.com.qmth.examcloud.core.questions.dao.*;
import cn.com.qmth.examcloud.core.questions.dao.entity.*;
import cn.com.qmth.examcloud.core.questions.service.*;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.*;
import cn.com.qmth.examcloud.core.questions.service.config.SysProperty;
import cn.com.qmth.examcloud.core.questions.service.export.ExportPaperAbstractService;
import com.google.gson.Gson;
import main.java.com.UpYun;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.docx4j.XmlUtils;
import org.docx4j.jaxb.Context;
import org.docx4j.wml.Body;
import org.docx4j.wml.P;
import org.docx4j.wml.R;
import org.docx4j.wml.Text;
import org.nlpcn.commons.lang.util.StringUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.domain.Sort.Order;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static cn.com.qmth.examcloud.core.questions.service.cache.Constants.CACHE_KEY_PAPER;
import static cn.com.qmth.examcloud.core.questions.service.cache.Constants.CACHE_KEY_QUESTION;

/**
 * @author chenken
 * @date 2017年9月12日 上午11:20:44
 * @company QMTH
 * @description PaperServiceImpl.java
 */
@Service("paperService")
public class PaperServiceImpl implements PaperService {

    protected ExamCloudLog log = ExamCloudLogFactory.getLog(this.getClass());

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailService paperDetailService;

    @Autowired
    ExamPaperRepo examPaperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    @Autowired
    QuesRepo quesRepo;

    @Autowired
    QuesBakRepo quesBakRepo;

    @Autowired
    Gson gson;

    @Autowired
    PaperDetailUnitService paperDetailUnitService;

    @Autowired
    QuesService quesService;

    @Autowired
    ExtractConfigService extractConfigService;

    @Autowired
    private MongoTemplate mongoTemplate;

    @Autowired
    private QuestionAudioServiceImpl questionAudioService;

    @Autowired
    private ExportServiceManageRepo exportServiceManageRepo;

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private CoursePaperCloudService coursePaperCloudService;

    @Autowired
    private CourseService courseService;

    private Random random = new Random();

    public static final String TEMP_FILE_EXP = "docxExport/";

    /**
     * 查询所有已导入试卷
     *
     * @param paperSearchInfo
     * @param curPage
     * @param pageSize
     * @return
     */
    public Page<Paper> getImportPapers(PaperSearchInfo paperSearchInfo, int curPage, int pageSize) {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(paperSearchInfo.getOrgId()));
        query.addCriteria(Criteria.where("paperType").is(PaperType.IMPORT));
        query.addCriteria(Criteria.where("course.enable").is("true"));
        if (StringUtils.isNotBlank(paperSearchInfo.getCourseNo())) {
            query.addCriteria(Criteria.where("course.code").is(paperSearchInfo.getCourseNo()));
        }
        if (StringUtils.isNotBlank(paperSearchInfo.getSpecialtyNo())) {
            query.addCriteria(Criteria.where("specialty.code").is(paperSearchInfo.getSpecialtyNo()));
        }
        if (StringUtils.isNotBlank(paperSearchInfo.getLevel())) {
            query.addCriteria(Criteria.where("course.level").is(paperSearchInfo.getLevel()));
        }
        if (StringUtils.isNoneBlank(paperSearchInfo.getName())) {
            String paperName = CommonUtils.escapeExprSpecialWord(paperSearchInfo.getName());
            query.addCriteria(Criteria.where("name").regex(".*?\\.*" + paperName + ".*"));
        }
        long count = this.mongoTemplate.count(query, Paper.class);
        query.with(new Sort(new Order(Direction.DESC, "createTime")));
        query.limit(pageSize);
        query.skip((curPage - 1L) * pageSize);
        List<Paper> paperList = this.mongoTemplate.find(query, Paper.class);
        return new PageImpl<Paper>(paperList, new PageRequest(curPage - 1, pageSize), count);
    }

    /**
     * 查询所有待审核和审核不通过的导入试卷
     *
     * @param paperSearchInfo
     * @param curPage
     * @param pageSize
     * @return
     */
    public Page<Paper> getImportPapersNotSuccess(PaperSearchInfo paperSearchInfo, int curPage, int pageSize) {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(paperSearchInfo.getOrgId()));
        query.addCriteria(Criteria.where("paperType").is(PaperType.IMPORT));
        query.addCriteria(Criteria.where("course.enable").is("true"));
        if (paperSearchInfo.getPaperStatus() != null) {
            query.addCriteria(Criteria.where("paperStatus").is(paperSearchInfo.getPaperStatus()));
        } else {
            query.addCriteria(Criteria.where("paperStatus").ne(PaperStatus.PASS));
        }
        if (StringUtil.isNotBlank(paperSearchInfo.getCourseNo())) {
            query.addCriteria(Criteria.where("course.code").is(paperSearchInfo.getCourseNo()));
        }
        long totalNumber = this.mongoTemplate.count(query, Paper.class);
        query.limit(pageSize);
        query.skip((curPage - 1L) * pageSize);
        List<Paper> paperList = this.mongoTemplate.find(query, Paper.class);
        return new PageImpl<Paper>(paperList, new PageRequest(curPage - 1, pageSize), totalNumber);
    }

    /**
     * 根据条件查询
     *
     * @param paperSearchInfo
     * @return
     */
    public List<Paper> getImportPapersBySearch(PaperSearchInfo paperSearchInfo) {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(paperSearchInfo.getOrgId()));
        query.addCriteria(Criteria.where("paperType").is(PaperType.IMPORT.name()));
        query.addCriteria(Criteria.where("course.enable").is("true"));
        if (StringUtils.isNotBlank(paperSearchInfo.getCourseNo())) {
            query.addCriteria(Criteria.where("course.code").is(paperSearchInfo.getCourseNo()));
        }
        if (StringUtils.isNotBlank(paperSearchInfo.getLevel())) {
            query.addCriteria(Criteria.where("course.level").is(paperSearchInfo.getLevel()));
        }
        List<Paper> paperList = this.mongoTemplate.find(query, Paper.class);
        return paperList;
    }

    /**
     * 根据条件查询
     *
     * @param paperSearchInfo
     * @return
     */
    public List<Paper> getGenPapersBySearch(PaperSearchInfo paperSearchInfo) {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(paperSearchInfo.getOrgId()));
        query.addCriteria(Criteria.where("paperType").is(PaperType.GENERATE.name()));
        query.addCriteria(Criteria.where("course.enable").is("true"));

        if (StringUtils.isNotBlank(paperSearchInfo.getCourseNo())) {
            query.addCriteria(Criteria.where("course.code").is(paperSearchInfo.getCourseNo()));
        }

        if (StringUtils.isNotBlank(paperSearchInfo.getLevel())) {
            query.addCriteria(Criteria.where("course.level").is(paperSearchInfo.getLevel()));
        }

        query.with(Sort.by(Sort.Direction.DESC, "createTime"));

        List<Paper> paperList = this.mongoTemplate.find(query, Paper.class);
        return paperList;
    }

    /**
     * 保存试卷
     *
     * @param paperExp
     * @return
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> savePaper(PaperExp paperExp, User user) {
        Map<String, Object> msgMap = new HashMap<>();

        Paper oldPaper = Model.of(paperRepo.findById(paperExp.getId()));
        if (oldPaper != null) {
            String oldName = oldPaper.getName().trim();
            oldPaper.setTitle(paperExp.getTitle());
            oldPaper.setName(paperExp.getName().trim());
            oldPaper.setLastModifyName(user.getDisplayName());
            oldPaper.setExamRemark(paperExp.getExamRemark());
            if (!oldName.equals(paperExp.getName().trim())) {// 假如改变了试卷名称
                // 则要效验试卷名称唯一性
                boolean existName = this.checkPaperName(paperExp.getName().trim(), user.getRootOrgId().toString());
                if (existName) {
                    msgMap.put("msg", "试卷名称重复，请重新命名！");
                } else {
                    formatPaper(oldPaper, user);
                    paperRepo.save(oldPaper);
                    msgMap.put("msg", "success");
                }
            } else {
                formatPaper(oldPaper, user);
                paperRepo.save(oldPaper);
                msgMap.put("msg", "success");
            }
        }

        //清除缓存
        this.clearPaperCache(paperExp.getId());

        return msgMap;
    }

    /**
     * 查询所有已组试卷
     *
     * @param paperSearchInfo
     * @param curPage
     * @param pageSize
     * @return
     */
    public Page<Paper> getGenPapers(PaperSearchInfo paperSearchInfo, int curPage, int pageSize) {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(paperSearchInfo.getOrgId()));
        query.addCriteria(Criteria.where("paperType").is(PaperType.GENERATE.name()));
        query.addCriteria(Criteria.where("course.enable").is("true"));
        if (StringUtils.isNotBlank(paperSearchInfo.getCourseNo())) {
            query.addCriteria(Criteria.where("course.code").is(paperSearchInfo.getCourseNo()));
        }
        if (StringUtils.isNoneBlank(paperSearchInfo.getName())) {
            String paperName = CommonUtils.escapeExprSpecialWord(paperSearchInfo.getName());
            query.addCriteria(Criteria.where("name").regex(".*?\\.*" + paperName + ".*"));
        }
        if (StringUtils.isNotBlank(paperSearchInfo.getLevel())) {
            query.addCriteria(Criteria.where("course.level").is(paperSearchInfo.getLevel()));
        }
        long count = this.mongoTemplate.count(query, Paper.class);
        query.with(new Sort(new Order(Direction.DESC, "createTime")));
        query.limit(pageSize);
        query.skip((curPage - 1L) * pageSize);
        List<Paper> paperList = this.mongoTemplate.find(query, Paper.class);
        return new PageImpl<Paper>(paperList, new PageRequest(curPage - 1, pageSize), count);
    }

    /**
     * 查询考试试卷
     *
     * @param id
     * @param courseCode
     * @param groupCode
     * @return
     */
    public List<Paper> listExamPapers(long id, String courseCode, String groupCode) {
        List<Paper> papers = new ArrayList<>();
        ExamPaper examPaper = new ExamPaper();
        examPaper.setExamId(id);
        examPaper.setCourseCode(courseCode);
        examPaper.setGroupCode(groupCode);
        Example<ExamPaper> example = Example.of(examPaper);
        List<ExamPaper> examPapers = examPaperRepo.findAll(example);
        for (ExamPaper ePaper : examPapers) {
            Paper paper = Model.of(paperRepo.findById(ePaper.getPaper().getId()));
            papers.add(paper);
        }
        return papers;
    }

    /**
     * 设置考试试卷
     *
     * @param examId
     * @param courseCode
     * @param groupCode
     * @param paperId
     * @return
     */
    public void joinToExamPaper(long examId, String courseCode, String groupCode, String paperId) {
        ExamPaper examPaper = new ExamPaper();
        examPaper.setExamId(examId);
        examPaper.setGroupCode(groupCode);
        examPaper.setCourseCode(courseCode);
        // examPaper.setPaperId(paperId);
        examPaperRepo.save(examPaper);
    }

    public void releaseExamPaper(long examId, String courseCode, String groupCode, String paperId) {
        ExamPaper examPaper = new ExamPaper();
        examPaper.setExamId(examId);
        examPaper.setGroupCode(groupCode);
        examPaper.setCourseCode(courseCode);
        // examPaper.setPaperId(paperId);
        examPaperRepo.delete(examPaper);
    }

    public Set<String> listGroupCodes(long examId, String courseCode) {
        Set<String> groupSet = new HashSet<>();
        ExamPaper examPaper = new ExamPaper();
        examPaper.setExamId(examId);
        examPaper.setCourseCode(courseCode);
        List<ExamPaper> examPapers = examPaperRepo.findAll(Example.of(examPaper));
        for (ExamPaper expaper : examPapers) {
            groupSet.add(expaper.getGroupCode());
        }
        return groupSet;
    }

    public void deletGroupCode(long examId, String courseCode, String groupCode) {
        ExamPaper examPaper = new ExamPaper();
        examPaper.setExamId(examId);
        examPaper.setCourseCode(courseCode);
        examPaper.setCourseCode(courseCode);
        examPaperRepo.delete(examPaper);
    }

    /**
     * 根据试卷ID获取试卷下面的大题
     *
     * @param id
     * @return
     */
    public List<PaperDetail> findPaperDetailsById(String id) {
        return paperDetailService.getPaperDetailsByPaper(Model.of(paperRepo.findById(id)));
    }

    /**
     * 批量删除试卷
     *
     * @param paperIds
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> deletePapers(List<String> paperIds) {
        Map<String, Object> msgMap = new HashMap<>();
        String msg = "";
        List<Paper> papers = CommonUtils.toList(paperRepo.findByIdIn(paperIds));
        if (papers.get(0).getPaperType() == PaperType.IMPORT) {
            List<Question> quesList = new ArrayList<>();
            for (Paper paper : papers) {
                List<PaperDetailUnit> paperUnits = paperDetailUnitRepo.findByPaperOrderByNumber(paper);
                for (PaperDetailUnit pdu : paperUnits) {
                    if (pdu.getQuestion() != null) {
                        quesList.add(pdu.getQuestion());
                    }
                }
            }
            List<PaperDetailUnit> allUnits = paperDetailUnitRepo.findByQuestionIn(quesList);
            for (PaperDetailUnit pdu : allUnits) {
                if (pdu.getPaper() != null && pdu.getPaper().getPaperType() == PaperType.GENERATE) {
                    msg = "待删除试卷中有试题被组卷使用，不能删除!";
                    msgMap.put("msg", msg);
                    msgMap.put("paperName", pdu.getPaper().getName());
                    return msgMap;
                }
            }
            //删除音频
            questionAudioService.deleteAudio(quesList);
            quesRepo.deleteAll(quesList);
        } else if (papers.get(0).getPaperType() == PaperType.GENERATE) {
            for (Paper paper : papers) {
                List<String> examPaperIds = extractConfigService.getExamPaperId(paper.getCourseNo(), paper.getOrgId());
                if (examPaperIds != null && examPaperIds.contains(paper.getId())) {
                    msg = "待删除试卷有被调卷规则使用，不能删除!";
                    msgMap.put("msg", msg);
                    msgMap.put("paperName", paper.getName());
                    return msgMap;
                }
            }
        }
        paperDetailService.deletePaperDetailsByPapers(papers);
        paperRepo.deleteAll(papers);

        for (String paperId : paperIds) {
            //清除缓存
            this.clearPaperCache(paperId);
        }

        msg = "success";
        msgMap.put("msg", msg);
        return msgMap;
    }

    /**
     * 批量通过试卷
     *
     * @param paperIds
     */
    public void passPapers(List<String> paperIds) {
        List<Paper> papers = CommonUtils.toList(paperRepo.findByIdIn(paperIds));
        papers.stream().forEach(paper -> {
            paper.setPaperStatus(PaperStatus.PASS);
        });
        paperRepo.saveAll(papers);
    }

    /**
     * 批量不通过试卷
     *
     * @param paperIds
     */
    public void noPassPapers(List<String> paperIds) {
        List<Paper> papers = CommonUtils.toList(paperRepo.findByIdIn(paperIds));
        papers.stream().forEach(paper -> {
            paper.setPaperStatus(PaperStatus.NOPASS);
        });
        paperRepo.saveAll(papers);
    }

    /**
     * 批量待审核试卷
     *
     * @param paperIds
     */
    public void backPapers(List<String> paperIds) {
        List<Paper> papers = CommonUtils.toList(paperRepo.findByIdIn(paperIds));
        papers.stream().forEach(paper -> {
            paper.setPaperStatus(PaperStatus.DRAFT);
        });
        paperRepo.saveAll(papers);
    }

    /**
     * 初始化导出试卷DTO
     *
     * @param id
     * @return
     */
    public PaperExp getPaperDto(String id) {
        Paper paper = Model.of(paperRepo.findById(id));
        // 创建paperDto
        PaperExp paperExp = new PaperExp();
        BeanUtils.copyProperties(paper, paperExp);
        paperExp.setCourse(paper.getCourse());
        // 获取大题
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
        List<PaperDetailExp> paperDetailExps = new ArrayList<>();
        for (PaperDetail paperDetail : paperDetails) {
            PaperDetailExp paperDetailExp = new PaperDetailExp();
            BeanUtils.copyProperties(paperDetail, paperDetailExp);
            paperDetailExps.add(paperDetailExp);
        }
        // 封装小题
        for (int i = 0; i < paperDetailExps.size(); i++) {
            List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetails.get(i));
            if (paperDetailUnits != null && paperDetailUnits.size() > 0) {
                List<PaperDetailUnitExp> paperDetailUnitExps = new ArrayList<>();
                for (PaperDetailUnit paperDetailUnit : paperDetailUnits) {
                    PaperDetailUnitExp paperDetailUnitExp = new PaperDetailUnitExp();
                    BeanUtils.copyProperties(paperDetailUnit, paperDetailUnitExp);
                    paperDetailUnitExps.add(paperDetailUnitExp);
                }
                // 选择题，套题下选择题 选项顺序重新排列
                reorderChoicequestionOption(paperDetailUnitExps);
                paperDetailExps.get(i).setPaperDetailUnits(paperDetailUnitExps);
            } else {
                paperDetailExps.get(i).setUnitCount(0);
            }
        }
        paperExp.setPaperDetails(paperDetailExps);
        // 初始化试卷内容
        initPaper(paperExp);
        return paperExp;
    }

    /**
     * 重新对选择题option进行排序(多选、单选、套题下选择题)
     */
    public void reorderChoicequestionOption(List<PaperDetailUnitExp> paperDetailUnitExps) {
        for (PaperDetailUnitExp paperDetailUnitExp : paperDetailUnitExps) {
            String optionOrder = paperDetailUnitExp.getOptionOrder();
            if (StringUtil.isNotBlank(optionOrder)) {
                Question question = paperDetailUnitExp.getQuestion();
                if (question.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                        || question.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                    question.setQuesOptions(reorderOptionCore(question.getQuesOptions(), optionOrder));
                }
                if (question.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                    List<Question> subQuestions = question.getSubQuestions();
                    int index = 0;
                    for (int k = 0; k < subQuestions.size(); k++) {
                        Question subQuestion = subQuestions.get(k);
                        if (subQuestion.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                                || subQuestion.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                            subQuestion.setQuesOptions(
                                    reorderOptionCore(subQuestion.getQuesOptions(), optionOrder.split(";")[index]));
                            index++;
                        }
                    }
                }
            }
        }
    }

    private List<QuesOption> reorderOptionCore(List<QuesOption> quesOptions, String optionOrder) {
        List<QuesOption> newQuesOptions = new ArrayList<>();
        if (StringUtil.isBlank(optionOrder) || quesOptions.isEmpty()) {
            return null;
        }
        String[] optionOrderArr = optionOrder.split(",");
        for (int j = 0; j < optionOrderArr.length; j++) {
            for (int k = 0; k < quesOptions.size(); k++) {
                if (optionOrderArr[j].equals(quesOptions.get(k).getNumber())) {
                    newQuesOptions.add(quesOptions.get(k));
                }
            }
        }
        quesOptions = null;
        return newQuesOptions;
    }

    /**
     * 初始化试卷内容(增加序号)
     *
     * @param paperExp
     */
    public void initPaper(PaperExp paperExp) {
        if (paperExp.getPaperDetails() == null || paperExp.getPaperDetails().size() == 0) {
            return;
        }
        int mainNum = 0;
        List<PaperDetailExp> paperDetailExpList = paperExp.getPaperDetails();
        for (PaperDetailExp paperDetail : paperDetailExpList) {
            // 大题序号
            paperDetail.setNumber(++mainNum);
            paperDetail.setCnNum(CommonUtils.toCHNum(paperDetail.getNumber()));
            if (paperDetail != null && paperDetail.getPaperDetailUnits() != null && paperDetail.getPaperDetailUnits().size() > 0) {
                for (PaperDetailUnitExp paperDetailUnit : paperDetail.getPaperDetailUnits()) {
                    Question question = paperDetailUnit.getQuestion();
                    if (question != null) {
                        if (question.getHasAudio() != null && question.getHasAudio()) {
                            paperExp.setHasAudio(true);    //设置试卷含有音频
                        }
                        quesService.formatQuesUnit(question);
                        List<Question> subQuesList = question.getSubQuestions();
                        // 套题序号
                        if (subQuesList != null && subQuesList.size() > 0) {
                            int index = 0;
                            for (Question subQues : subQuesList) {
                                Map<String, String> params = new HashMap<>();
                                params.put("number", String.valueOf(++index));
                                subQues.setQuesParams(params);
                                quesService.formatQuesUnit(subQues);
                            }
                            String quesBodyHtml = CommonUtils.relaceQuestionIdx(question.getQuesBody(), 0);
                            question.setQuesBody(quesBodyHtml);
                        }
                    }
                }
            }
        }
    }

    /**
     * 格式化查询条件
     *
     * @param paperSearchInfo
     */
    public void formatPaperSearchInfo(PaperSearchInfo paperSearchInfo) {
        if (StringUtils.isEmpty(paperSearchInfo.getCourseNo())) {
            paperSearchInfo.setCourseNo(null);
        }
        if (StringUtils.isEmpty(paperSearchInfo.getCreateTime())) {
            paperSearchInfo.setCreateTime(null);
        }
        if (StringUtils.isEmpty(paperSearchInfo.getCreator())) {
            paperSearchInfo.setCreator(null);
        }
        if (StringUtils.isEmpty(paperSearchInfo.getName())) {
            paperSearchInfo.setName(null);
        }
    }

    /**
     * 1.重新设置小题number，并返回试卷总分
     * 2.重新计算大题的总分和小题总数
     * 3.重新保存试卷大题数量、小题数量、试卷总分等信息
     *
     * @param paper
     */
    public void formatPaper(Paper paper, User user) {
        double paperTotalScore = reSetPaperDetailUnit(paper);
        paper.setDifficultyDegree(reSetDifficulty(paper, paperTotalScore));
        Map<String, Object> paperInfoMap = reSetPaperDetail(paper);
        reSetPaper(paper, user, paperInfoMap, paperTotalScore);
    }

    /**
     * 计算试卷难度
     *
     * @param paper
     * @param paperTotalScore
     * @return
     */

    @SuppressWarnings("unused")
    public Double reSetDifficulty(Paper paper, double paperTotalScore) {
        Double sum = 0.0;
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
        if (paperDetails != null && paperDetails.size() > 0) {
            for (PaperDetail paperDetail : paperDetails) {
                //获取每个大题下面的所有小题
                List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetail);
                if (paperDetailUnits != null && paperDetailUnits.size() > 0) {
                    for (PaperDetailUnit paperDetailUnit : paperDetailUnits) {
                        //如果为套题，重新计算难度
                        if (paperDetailUnit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                            //重新计算套题的难度，公开度
                            boolean publicity = false;
                            double totalSum = 0d;
                            double totalDou = 0d;
                            for (int i = 0; i < paperDetailUnit.getQuestion().getSubQuestions().size(); i++) {
                                Question subQuestion = paperDetailUnit.getQuestion().getSubQuestions().get(i);
                                //设置公开度
                                if (subQuestion.getPublicity() == null) {
                                    publicity = true;
                                } else {
                                    if (subQuestion.getPublicity()) {
                                        publicity = true;
                                    }
                                }
                                if (subQuestion.getDifficultyDegree() == null) {
                                    subQuestion.setDifficultyDegree(0.5);
                                }
                                totalSum = subQuestion.getDifficultyDegree() * paperDetailUnit.getSubScoreList().get(i) + totalSum;
                                totalDou = paperDetailUnit.getSubScoreList().get(i) + totalDou;
                            }

                            BigDecimal b;
                            if (totalDou != 0d) {
                                b = BigDecimal.valueOf(totalSum / totalDou);
                            } else {
                                b = BigDecimal.valueOf(0d);
                            }

                            Double difficulty = b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                            paperDetailUnit.getQuestion().setDifficultyDegree(difficulty);
                            paperDetailUnit.setScore(totalDou);
                        }
                        //旧题没有难度值，需要重新赋值
                        if (paperDetailUnit.getQuestion().getDifficultyDegree() == null) {
                            paperDetailUnit.getQuestion().setDifficultyDegree(0.5);
                        }
                        if (paperDetailUnit.getScore() == null) {
                            paperDetailUnit.setScore(0d);
                        }
                        sum = paperDetailUnit.getScore() * paperDetailUnit.getQuestion().getDifficultyDegree() + sum;
                    }
                }
            }
            if (paperTotalScore < 0.1) {
                return 0.0;
            }
            BigDecimal b = BigDecimal.valueOf(sum / paperTotalScore);
            Double difficulty = b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            return difficulty;
        }
        return 0.0;
    }

    /**
     * 重新设置小题number，并返回试卷总分
     *
     * @param paper
     * @return
     */
    private double reSetPaperDetailUnit(Paper paper) {
        List<PaperDetailUnit> paperDetailUnitAll = paperDetailUnitService.findByPaperAndSort(paper);
        double totalScore = 0;
        for (int i = 0; i < paperDetailUnitAll.size(); i++) {
            PaperDetailUnit unit = paperDetailUnitAll.get(i);
            if (unit.getScore() != null) {
                totalScore += unit.getScore();
            }
            unit.setNumber(i + 1);
        }
        paperDetailUnitRepo.saveAll(paperDetailUnitAll);
        totalScore = CommonUtils.formatDouble(totalScore);
        return totalScore;
    }

    /**
     * 重新计算大题的总分和小题总数
     *
     * @param paper
     */
    private Map<String, Object> reSetPaperDetail(Paper paper) {
        Map<String, Object> paperInfoMap = new HashMap<>();
        int allQuesCount = 0;
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
        // 计算各大题总分和小题数量
        for (PaperDetail paperDetail : paperDetails) {
            List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetail);
            if (paperDetailUnits.size() > 0) {
                int count = 0;
                double score = 0;
                int nestQusNum = 0;
                for (PaperDetailUnit unit : paperDetailUnits) {
                    if (unit.getScore() != null) {
                        score += unit.getScore();
                    }
                    if (unit.getQuestion() != null
                            && unit.getQuestion().getSubQuestions() != null
                            && unit.getQuestion().getSubQuestions().size() > 0) {
                        nestQusNum += unit.getQuestion().getSubQuestions().size() - 1;
                    }
                }
                count = paperDetailUnits.size() + nestQusNum;
                score = CommonUtils.formatDouble(score);
                paperDetail.setScore(score);
                paperDetail.setUnitCount(count);
                allQuesCount += count;
            } else {
                paperDetail.setScore(0d);
            }
        }
        paperDetailRepo.saveAll(paperDetails);
        paperInfoMap.put("allQuesCount", allQuesCount);
        paperInfoMap.put("paperDetails", paperDetails);
        return paperInfoMap;
    }

    /**
     * 重新设置试卷大题数量、小题数量、试卷总分等属性
     *
     * @param paper
     * @param user
     * @param paperInfoMap
     * @param paperTotalScore
     */
    @SuppressWarnings("unchecked")
    private void reSetPaper(Paper paper, User user, Map<String, Object> paperInfoMap, double paperTotalScore) {
        List<PaperDetail> paperDetails = (List<PaperDetail>) paperInfoMap.get("paperDetails");
        paper.setPaperDetailCount(paperDetails.size());//设置大题数量
        paper.setUnitCount(Integer.parseInt(paperInfoMap.get("allQuesCount") + ""));//设置小题数量
        paper.setTotalScore(paperTotalScore);//设置试卷总分
        if (user != null) {
            paper.setLastModifyName(user.getDisplayName());
        }
        paperRepo.save(paper);

        //清除缓存
        this.clearPaperCache(paper.getId());
    }

    /**
     * 先备份准备删掉的试题，然后再删掉
     *
     * @param questionId
     * @return
     */
    public List<String> deleteImportQuestionById(String questionId, User user) {
        Question ques = Model.of(quesRepo.findById(questionId));

        List<PaperDetailUnit> pdus = CommonUtils.toList(paperDetailUnitRepo.findByQuestion(ques));
        List<String> paperNames = new ArrayList<>();

        // 需要删除的小题
        List<PaperDetailUnit> needPdus = new ArrayList<>();

        List<Paper> papers = new ArrayList<>();
        for (PaperDetailUnit pdu : pdus) {
            if (pdu.getPaper() != null) {
                if (!papers.contains(pdu.getPaper())) {
                    papers.add(pdu.getPaper());
                }
                if (PaperType.GENERATE == pdu.getPaper().getPaperType()) {
                    paperNames.add(pdu.getPaper().getName());
                }
            }
        }

        if (paperNames.size() == 0) {
            needPdus.addAll(pdus);// 此试题没有被组卷调用，则可以删除此试题
            paperDetailUnitRepo.deleteAll(needPdus);
            quesBakRepo.save(BeanCopierUtil.copyProperties(ques, QuestionBak.class));
            quesRepo.delete(ques);

            for (Paper paper : papers) {
                formatPaper(paper, user);
            }
        }

        return paperNames;
    }

    /**
     * 向试卷中插入一个试题
     *
     * @param paperId
     * @param paperDetailId
     * @return
     */
    public Paper insertQuestionToPaper(String paperId, String paperDetailId, Question question, User user) {
        Paper paper = Model.of(paperRepo.findById(paperId));
        question.setOrgId(user.getRootOrgId().toString());
        question.setCourse(paper.getCourse());
        question.setCourseNo(paper.getCourse().getCode());//必须设置，因为部分方法需要courseNo作为查询条件
        question = quesService.saveQues(question);

        PaperDetail paperDetail = Model.of(paperDetailRepo.findById(paperDetailId));
        PaperDetailUnit paperDetailUnit = new PaperDetailUnit(paper, paperDetail, question);
        paperDetailUnit.setPaperType(PaperType.IMPORT);
        PaperDetailUnit paperDetailUnit2 = paperDetailUnitService.findTopOrderByNumber(paperDetail, "DESC");
        if (paperDetailUnit2 == null) {
            paperDetailUnit.setNumber(1);
        } else {
            paperDetailUnit.setNumber(paperDetailUnit2.getNumber());//设置number为大题的最大题号
        }
        paperDetailUnitRepo.save(paperDetailUnit);
        formatPaper(paper, user);
        return paper;
    }

    /**
     * 获取试题所在的试卷名称
     *
     * @param questionId
     * @return
     */
    public List<String> getPaperNamesByQuestionId(String questionId) {
        List<String> paperNames = new ArrayList<>();
        List<PaperDetailUnit> pdus = paperDetailUnitRepo.findByQuestion(Model.of(quesRepo.findById(questionId)));
        for (PaperDetailUnit pdu : pdus) {
            paperNames.add(pdu.getPaper().getName());
        }
        return paperNames;

    }

    public Page<Question> listQuestionforSelect(String paperId, int curPage, int pageSize, QuesStructType quesType, User user, String quesBody) {
        Set<String> selectedIds = new HashSet<>();
        Paper paper = Model.of(paperRepo.findById(paperId));
        List<PaperDetailUnit> pdus = paperDetailUnitRepo.findByPaperOrderByNumber(paper);
        for (PaperDetailUnit pdu : pdus) {
            selectedIds.add(pdu.getQuestion().getId());
        }
        return quesService.findByIdExclude(selectedIds, paper.getCourseNo(), quesType, curPage, pageSize, user.getRootOrgId(), quesBody);
    }

    @SuppressWarnings("unused")
    public Paper selectQuestionsToPaper(String paperId, String paperDetailId, List<Question> questions, User user) {
        Paper paper = Model.of(paperRepo.findById(paperId));
        PaperDetail paperDetail = Model.of(paperDetailRepo.findById(paperDetailId));
        PaperDetailUnit paperDetailUnit = paperDetailUnitRepo.findTopByPaperDetailOrderByNumberDesc(paperDetail);

        List<PaperDetailUnit> saveUnits = new ArrayList<>();
        for (Question ques : questions) {
            PaperDetailUnit pdu = new PaperDetailUnit(paper, paperDetail, ques);

            //如果大题没有小题，取导入的试题分数
            if (paperDetailUnit == null) {
                pdu.setNumber(1);
            } else {
                pdu.setNumber(paperDetailUnit.getNumber());//设置为大题中最大的number
            }

            pdu.setScore(ques.getScore() == null ? 0d : ques.getScore());

            //处理套题
            if (pdu.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                List<Question> subQuestions = ques.getSubQuestions();
                List<Double> subScoreList = new ArrayList<>();
                if (subQuestions != null && subQuestions.size() > 0) {
                    for (Question subQuestion : subQuestions) {
                        subScoreList.add(CommonUtils.formatDouble(pdu.getScore() / subQuestions.size()));
                    }
                }
                pdu.setSubScoreListNew(subScoreList);
            }

            saveUnits.add(pdu);
        }

        paperDetailUnitRepo.saveAll(saveUnits);

        //清除缓存
        this.clearPaperCache(paper.getId());

        formatPaper(paper, user);
        return paper;
    }

    public boolean checkPaperName(String paperName, String orgId) {
        List<Paper> paperList = paperRepo.findByNameAndOrgId(paperName, orgId);
        if (CollectionUtils.isNotEmpty(paperList)) {
            return true;
        }
        return false;
    }

    public void checkPaperNameNew(String paperName, String orgId) throws Exception {
        List<Paper> paperList = paperRepo.findByNameAndOrgId(paperName, orgId);
        if (paperList != null && paperList.size() > 0) {
            throw new PaperException("试卷名称重复，请重新命名");
        }
    }

    public Page<Paper> getPapersNotInIds(PaperSearchInfo paperSearchInfo, String[] ids, int curPage, int pageSize, PaperType paperType) {
        Set<String> selectedIds = new HashSet<>();
        for (String id : ids) {
            selectedIds.add(id);
        }
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(paperSearchInfo.getOrgId()));
        query.addCriteria(Criteria.where("paperType").is(paperType));
        query.addCriteria(Criteria.where("course.code").is(paperSearchInfo.getCourseNo()));
        query.addCriteria(Criteria.where("id").nin(selectedIds));
        long total = this.mongoTemplate.count(query, Paper.class);
        query.with(new Sort(new Order(Direction.DESC, "createTime")));
        query.limit(pageSize);
        query.skip((curPage - 1L) * pageSize);
        List<Paper> paperList = this.mongoTemplate.find(query, Paper.class);
        Page<Paper> paperPageList = new PageImpl<Paper>(paperList, new PageRequest(curPage - 1, pageSize), total);
        return paperPageList;
    }

    /**
     * 将选中的导入试卷复制为卷库试卷
     */
    public void useBasePaper(String selectedPaperIds, String userId) {
        Assert.hasLength(selectedPaperIds, "试卷id不能为空!");
        String[] paperIds = selectedPaperIds.split(",");
        for (int i = 0; i < paperIds.length; i++) {
            Paper oldpaper = Model.of(paperRepo.findById(paperIds[i]));
            List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(oldpaper);
            oldpaper.setId(null);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String createTime = sdf.format(new Date());
            oldpaper.setCreateTime(createTime);
            oldpaper.setCreator(userId);
            oldpaper.setLastModifyName(userId);
            oldpaper.setPaperType(PaperType.GENERATE);//试卷类型 修改为组卷
            Paper newPaper = paperRepo.save(oldpaper);
            for (int j = 0; j < paperDetails.size(); j++) {
                PaperDetail paperDetail = paperDetails.get(j);
                List<PaperDetailUnit> paperDetailUnits = paperDetailUnitService.getUnitsByPaperDetail(paperDetail);
                paperDetail.setPaper(newPaper);//关联新Paper
                paperDetail.setId(null);
                paperDetail.setCreateTime(createTime);
                paperDetail.setCreator(userId);
                PaperDetail newPaperDetail = paperDetailRepo.save(paperDetail);//保存新的paperDetail
                for (int k = 0; k < paperDetailUnits.size(); k++) {
                    //重新设置保存PaperDetailUnit
                    PaperDetailUnit paperDetailUnit = paperDetailUnits.get(k);
                    paperDetailUnit.setPaper(newPaper);                //关联新Paper
                    paperDetailUnit.setPaperDetail(newPaperDetail); //关联新paperDetail
                    paperDetailUnit.setId(null);
                    paperDetailUnit.setCreateTime(createTime);
                    paperDetailUnit.setCreator(userId);
                    paperDetailUnit.setPaperType(PaperType.GENERATE);
                    paperDetailUnitRepo.save(paperDetailUnit);//保存新的paperDetailUnit
                }
            }
        }
    }

    /**
     * 根据试卷名称、试卷类型检查名称是否存在
     *
     * @param paperName
     * @param paperType
     * @param orgId
     * @return
     * @throws Exception
     */
    public boolean checkPaperName(String paperName, PaperType paperType, String orgId) throws Exception {
        Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(orgId));
        query.addCriteria(Criteria.where("name").is(paperName.trim()));
        query.addCriteria(Criteria.where("paperType").is(paperType));
        List<Paper> papers = this.mongoTemplate.find(query, Paper.class);
        if (papers != null && papers.size() > 0) {
            return false;
        }
        return true;
    }

    /**
     * 上传音频文件检查
     *
     * @param paperId
     * @param filesName
     * @return
     */
    public Map<String, String> checkRadioFile(String paperId, List<String> filesName) {
        Map<String, String> messageMap = new HashMap<>();
        //判断文件名中格式是否正确
        for (String fileName : filesName) {
            String[] fileNames = fileName.split("\\.");
            String fileType = fileNames[fileNames.length - 1];
            if (sysProperty.getRadioType().indexOf(fileType) < 0) {
                messageMap.put("errorMsg", fileName + ",文件格式不正确,当前支持格式:" + sysProperty.getRadioType());
                return messageMap;
            }

            String pattern_01 = "\\d+_1_\\d{1,2}";                //题干正则
            String pattern_02 = "\\d+_2_[A-Z|a-z]_\\d{1,2}";    //选项正则
            if (!(Pattern.matches(pattern_01, fileNames[0]) || Pattern.matches(pattern_02, fileNames[0]))) {
                messageMap.put("errorMsg", fileName + "文件名格式不正确,请检查");
                return messageMap;
            }
        }

        //根据试卷id，查询该试卷
        Paper paper = Model.of(paperRepo.findById(paperId));

        //根据试卷查询所有的小题
        List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperOrderByNumber(paper);

        String names = "";
        for (PaperDetailUnit paperDetailUnit : paperDetailUnits) {
            names = names + paperDetailUnit.getNumber().toString() + ",";
        }

        for (String fileName : filesName) {
            String fileNames[] = fileName.split("_");

            //先判断小题号是否正确
            String fileNameFirst = fileNames[0];
            if (!names.contains(fileNameFirst)) {
                messageMap.put("errorMsg", fileName + "文件，试卷中没有对应的小题");
                return messageMap;
            }

            //再判断题干中是否存在ABCD
            String fileNameSecond = fileNames[1];
            if (fileNameSecond.equals("1")) {
                if (fileNames.length > 3) {
                    Matcher m = Pattern.compile(".*[a-zA-Z]+.*").matcher(fileNames[2]);
                    if (m.matches()) {
                        messageMap.put("errorMsg", fileName + "文件名称不对，文件名为题干，但存在选项");
                        return messageMap;
                    }
                }
            }

            //判断选项
            else if (fileNameSecond.equals("2")) {
                messageMap = checkOptions(paperDetailUnits, fileNames, fileName);
                if (messageMap != null) {
                    return messageMap;
                }
                messageMap = new HashMap<>();
            } else {
                messageMap.put("errorMsg", fileName + "文件名称不对，无法识别为题干或选项 ");
                return messageMap;
            }
        }

        messageMap.put("errorMsg", "OK");
        return messageMap;
    }

    //判断选项
    public Map<String, String> checkOptions(List<PaperDetailUnit> paperDetailUnits, String fileNames[], String fileName) {
        Map<String, String> messageMap = new HashMap<>();
        for (PaperDetailUnit paperDetailUnit : paperDetailUnits) {
            if (paperDetailUnit.getNumber().toString().equals(fileNames[0])) {
                //判断是否为选择题
                if (paperDetailUnit.getQuestionType() != QuesStructType.SINGLE_ANSWER_QUESTION
                        && paperDetailUnit.getQuestionType() != QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                    Matcher m = Pattern.compile(".*[a-zA-Z]+.*").matcher(fileNames[2]);
                    if (m.matches()) {
                        messageMap.put("errorMsg", fileName + "文件名称有误,题目不是选择题");
                        return messageMap;
                    }
                    break;
                } else {
                    List<QuesOption> options = paperDetailUnit.getQuestion().getQuesOptions();
                    String option = "";
                    for (QuesOption quesOption : options) {
                        option = option + quesOption.getNumber() + ",";
                    }
                    Integer integer = CommonUtils.characterToNumber(fileNames[2]);
                    if (!option.contains(integer.toString())) {
                        messageMap.put("errorMsg", fileName + "文件名称有误，题目中没有对应的" + fileNames[2] + "选项");
                        return messageMap;
                    }
                    break;
                }
            }
        }
        return null;
    }

    /**
     * 上传音频文件到又拍云
     *
     * @param files
     * @param paperId
     * @throws IOException
     */
    public void uploadRadio(List<MultipartFile> files, String paperId, User user) {
        //根据试卷id，查询该试卷
        Paper paper = Model.of(paperRepo.findById(paperId));

        for (MultipartFile file : files) {
            //判断文件大小
            long fileSize = file.getSize();
            int size = Integer.parseInt(sysProperty.getAudioMaxsize());
            if (fileSize > (size * 1048576L)) {
                throw new StatusException("400", "音频文件过大，限制5M内！");
            }

            //根据试卷查询所有的小题，根据文件名匹配出当前小题ID
            String numbers[] = file.getOriginalFilename().split("_");
            if (numbers.length < 3) {
                throw new StatusException("400", "音频文件命名格式错误！");
            }

            PaperDetailUnit unit = paperDetailUnitRepo.findByPaperAndNumber(paper, Integer.valueOf(numbers[0]));
            if (unit == null || unit.getQuestion() == null) {
                throw new StatusException("400", "对应试题不存在！");
            }

            Question question = unit.getQuestion();
            uploadAudioFile(paperId, question.getId(), file, user);
            appendAudioTag(file.getOriginalFilename(), question.getId());

            //清除缓存
            this.clearQuestionCache(question.getId());
        }

        //删除服务器文件夹
        String mp3DirectoryPath = TEMP_FILE_EXP + File.separator + paperId;
        try {
            File mp3Directory = new File(mp3DirectoryPath);
            FileUtils.deleteDirectory(mp3Directory);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 上传音频文件至又拍云
     */
    private void uploadAudioFile(String paperId, String questionId, MultipartFile file, User user) {
        //文件夹不存在则新建
        final String mp3DirectoryPath = TEMP_FILE_EXP + File.separator + paperId;
        File mp3Directory = new File(mp3DirectoryPath);
        if (!mp3Directory.exists()) {
            mp3Directory.mkdirs();
        }

        //文件名包含随机数，防止缓存
        int randomNumber = random.nextInt(1000);
        final String mp3FileName = questionId + "_" + randomNumber + "_" + file.getOriginalFilename();
        File mp3File = new File(mp3DirectoryPath + File.separator + mp3FileName);

        FileOutputStream outputStream;
        try {
            outputStream = new FileOutputStream(mp3File);
        } catch (FileNotFoundException e) {
            log.error(e.getMessage(), e);
            throw new StatusException("500", "音频文件处理失败！");
        }

        try (InputStream is = file.getInputStream();
             BufferedInputStream bis = new BufferedInputStream(is, 1024 * 10);) {
            int read;
            byte[] bytes = new byte[1024 * 4];
            while ((read = bis.read(bytes, 0, 1024 * 4)) != -1) {
                outputStream.write(bytes, 0, read);
            }

            //上传到又拍云
            UpYun upYun = new UpYun(sysProperty.getBucketName(), sysProperty.getUserName(), sysProperty.getPassword());
            upYun.writeFile(sysProperty.getRadioUploadPath() + mp3FileName, mp3File, true);
            IoUtils.removeFile(mp3File);

            //保存记录
            QuestionAudio audio = new QuestionAudio(questionId, file.getOriginalFilename(), sysProperty.getRadioUploadPath() + mp3FileName);
            questionAudioService.saveQuestionAudio(audio, user);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new StatusException("500", "音频文件保存失败！");
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException e) {
                //ignore
            }
        }
    }

    /**
     * 音频文件插入到标签
     */
    private void appendAudioTag(String fileName, String questionId) {
        QuestionAudio questionAudio = questionAudioService.findByQuestionIdAndFileName(questionId, fileName);
        if (questionAudio == null) {
            return;
        }

        Question question = Model.of(quesRepo.findById(questionAudio.getQuestionId()));

        //正则匹配音频标签
        Pattern audioPattern = Pattern.compile(String.format("<a.*name=\"%s\"></a>", fileName));
        final String audioTag = String.format("<a id=\"%s\" name=\"%s\"></a>", questionAudio.getId(), fileName);

        String numbers[] = fileName.split("_");
        if (numbers[1].equals("1")) {
            //处理题干
            String quesBody = question.getQuesBody();
            if (StringUtils.isBlank(quesBody)) {
                question.setQuesBody("<p>" + audioTag + "</p>");
            } else {
                Matcher matcher = audioPattern.matcher(quesBody);
                if (matcher.find()) {
                    //已存在音频标签则直接替换
                    question.setQuesBody(matcher.replaceAll(audioTag));
                } else {
                    //不存在音频标签则添加
                    question.setQuesBody(quesBody + "<p>" + audioTag + "</p>");
                }
            }
        } else {
            //处理选项
            for (QuesOption quesOption : question.getQuesOptions()) {
                Integer optNumber = CommonUtils.characterToNumber(numbers[2]);
                if (!quesOption.getNumber().equals(optNumber.toString())) {
                    continue;
                }

                String optionBody = quesOption.getOptionBody();
                if (StringUtils.isBlank(optionBody)) {
                    quesOption.setOptionBody("<p>" + audioTag + "</p>");
                    continue;
                }

                Matcher matcher = audioPattern.matcher(optionBody);
                if (matcher.find()) {
                    //已存在音频标签则直接替换
                    quesOption.setOptionBody(matcher.replaceAll(audioTag));
                } else {
                    //不存在音频标签则添加
                    quesOption.setOptionBody(optionBody + "<p>" + audioTag + "</p>");
                }
            }
        }

        question.setHasAudio(true);
        quesRepo.save(question);
    }

    @Override
    public Map<String, Object> getPaperPDF(String paperId, String type) {
        PaperExp paperExp = getPaperDto(paperId);
        Map<String, Object> map = new HashMap<>();
        map.put("courseName", paperExp.getCourseName());
        map.put("courseNo", paperExp.getCourseNo());
        List<String> htmlList = new ArrayList<>();
        if (paperExp.getPaperDetails() != null && paperExp.getPaperDetails().size() > 0) {
            for (PaperDetailExp paperDetail : paperExp.getPaperDetails()) {
                //添加大题标题
                if (StringUtils.isBlank(paperDetail.getTitle())) {
                    //htmlList.add(("<p>"+paperDetail.getCnNum()+"、"+paperDetail.getName()+"</p>").replaceAll("'", "&apos"));
                    htmlList.add(("<p class='paperDetailTitle'>" + paperDetail.getCnNum() + "、" + paperDetail.getName() + "</p>").replaceAll("'", "&apos"));
                } else {
                    htmlList.add("<p class='paperDetailTitle'>" + paperDetail.getCnNum() + "、" + paperDetail.getName() + paperDetail.getTitle() + "</p>");
                }
                if (paperDetail.getPaperDetailUnits() != null && paperDetail.getPaperDetailUnits().size() > 0) {
                    for (PaperDetailUnitExp paperDetailUnitExp : paperDetail.getPaperDetailUnits()) {
                        Question question = paperDetailUnitExp.getQuestion();
                        if (type.equals("paper")) {
                            //添加题干
                            String questionBody = CommonUtils.deleteHtmlP(paperDetailUnitExp.getNumber() + "." + question.getQuesBody() + "(" + paperDetailUnitExp.getScore() + "分)");
                            htmlList.add("<p class='questionBody'>" + questionBody + "</p>");
                            //判断是否为选择题
                            if (paperDetailUnitExp.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION || paperDetailUnitExp.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                                for (QuesOption quesOption : question.getQuesOptions()) {
                                    //添加选项
                                    String questionOption = CommonUtils.deleteHtmlP(CommonUtils.getOptionNum(Integer.parseInt(quesOption.getNumber()) - 1) + "." + quesOption.getOptionBody());
                                    htmlList.add("<p class='selectOption'>" + questionOption + "</p>");
                                }
                            }
                            //判断是否为套题
                            if (paperDetailUnitExp.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                                //得到子题
                                List<Question> subQuestions = question.getSubQuestions();
                                for (Question subQuestion : subQuestions) {
                                    //添加子题题干
                                    String subQuestionBody = CommonUtils.deleteHtmlP(subQuestion.getQuesParams().get("number") + "." + subQuestion.getQuesBody() + "(" + subQuestion.getScore() + "分)");
                                    htmlList.add("<p class='questionBody'>" + subQuestionBody + "</p>");
                                    //判断是否为选择题
                                    if (subQuestion.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION || subQuestion.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                                        for (QuesOption quesOption : subQuestion.getQuesOptions()) {
                                            //添加选项
                                            String questionOption = CommonUtils.deleteHtmlP(CommonUtils.getOptionNum(Integer.parseInt(quesOption.getNumber()) - 1) + "." + quesOption.getOptionBody());
                                            htmlList.add("<p class='selectOption'>" + questionOption + "</p>");
                                        }
                                    }
                                }
                            }
                        } else {
                            //判断是否为套题
                            if (paperDetailUnitExp.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                                //得到子题
                                List<Question> subQuestions = question.getSubQuestions();
                                for (Question subQuestion : subQuestions) {
                                    //添加答案
                                    String questionAnswer = CommonUtils.deleteHtmlP(subQuestion.getQuesParams().get("number") + "." + subQuestion.getQuesAnswer() + "(" + subQuestion.getScore() + "分)");
                                    htmlList.add("<p class='questionAnswer'>" + questionAnswer + "</p>");
                                }
                            } else {
                                //添加答案
                                String questionAnswer = CommonUtils.deleteHtmlP(paperDetailUnitExp.getNumber() + "." + question.getQuesAnswer() + "(" + paperDetailUnitExp.getScore() + "分)");
                                htmlList.add("<p class='questionAnswer'>" + questionAnswer + "</p>");
                            }
                        }

                    }
                }
            }
        }
        map.put("htmlList", htmlList);
        return map;
    }

    @SuppressWarnings("unused")
    @Override
    public String sendPrint(String paperId, String orgId, String examId) {
        //查询原paper对象
        Paper paper = Model.of(paperRepo.findById(paperId));
        SyncCoursePaperBean bean = new SyncCoursePaperBean();
        bean.setOrgId(Long.valueOf(orgId));
        bean.setExamId(Long.valueOf(examId));
        Course course = courseService.getCourse(Long.valueOf(orgId), paper.getCourse().getCode());
        bean.setCourseId(Long.valueOf(course.getId()));
        bean.setCourseCode(paper.getCourse().getCode());
        bean.setCourseName(paper.getCourse().getName());
        bean.setPaperId(paper.getId());
        bean.setPaperName(paper.getName());
        SyncCoursePaperReq req = new SyncCoursePaperReq();
        req.setBean(bean);
        SyncCoursePaperResp resp = coursePaperCloudService.syncCoursePaper(req);
        return "success";
    }

    @Override
    public String findQuestionStructure(String paperId) throws Exception {
        //根据paperId查询新的对象
        ExportServiceManage esm = exportServiceManageRepo.findByOrgName("陕西师范大学");
        ExportPaperAbstractService exportPaperAbstractService = (ExportPaperAbstractService) SpringContextUtils.getBeanById(esm.getExportServiceName());
        PaperExp paperExp = exportPaperAbstractService.initPaperExp(paperId);
        log.info("已经获取试卷");
        //得到试卷中的客观大题
        List<PaperDetailExp> objectiveDetails = exportPaperAbstractService.getAllObjectiveDetails(paperExp);
        //客观题中如果有套题，要拆开
        List<PaperDetailExp> newObjectiveDetails = buildPaperDetailExp(objectiveDetails);
        //生成客观题数集合
        List<ObjectiveQuestionStructure> objectiveQuestionStructureList = new ArrayList<>();
        for (PaperDetailExp paperDetailExp : newObjectiveDetails) {
            for (PaperDetailUnitExp unit : paperDetailExp.getPaperDetailUnits()) {
                objectiveQuestionStructureList.add(new ObjectiveQuestionStructure(paperExp, paperDetailExp, unit, null));
            }
        }
        log.info("生成客观题数集合");
        //得到试卷中的主观大题
        List<PaperDetailExp> subjectiveDetails = exportPaperAbstractService.getAllSubjectiveDetails(paperExp);
        //生成主观题数集合
        List<SubjectiveQuestionStructure> subjectiveQuestionStructureList = new ArrayList<>();
        for (PaperDetailExp paperDetailExp : subjectiveDetails) {
            for (PaperDetailUnitExp unit : paperDetailExp.getPaperDetailUnits()) {
                subjectiveQuestionStructureList.add(new SubjectiveQuestionStructure(paperExp, paperDetailExp, unit, null));
            }
        }
        log.info("生成主观题数集合");
        PaperQuestionStructureInfo info = new PaperQuestionStructureInfo();
        info.setObjectives(objectiveQuestionStructureList);
        info.setSubjectives(subjectiveQuestionStructureList);
        String jsonResp = JsonUtil.toJson(info);
        log.info("jsonResp:" + jsonResp);
        return jsonResp;
    }

    public List<PaperDetailExp> buildPaperDetailExp(List<PaperDetailExp> objectiveDetails) {
        //1.得到所有客观题大题的类型
        Set<QuesStructType> types = new HashSet<>();
        for (PaperDetailExp paperDetailExp : objectiveDetails) {
            types.add(QuesStructType.getQuesStructTypeById(paperDetailExp.getSortNumber()));
        }
        //2.生成新的大题集合
        List<PaperDetailExp> newObjs = new ArrayList<>();
        for (QuesStructType type : types) {
            for (PaperDetailExp paperDetailExp : objectiveDetails) {
                if (paperDetailExp.getSortNumber().equals(type.getId())) {
                    //生成新的大题
                    PaperDetailExp newPaperDetailExp = new PaperDetailExp();
                    //生成新的小题
                    List<PaperDetailUnitExp> newPaperDetailUnits = new ArrayList<>();
                    //得到旧大题下的所有小题
                    List<PaperDetailUnitExp> paperDetailUnits = paperDetailExp.getPaperDetailUnits();
                    for (PaperDetailUnitExp paperDetailUnitExp : paperDetailUnits) {
                        if (paperDetailUnitExp.getQuestionType() != QuesStructType.NESTED_ANSWER_QUESTION) {
                            newPaperDetailUnits.add(paperDetailUnitExp);
                        } else {
                            List<Question> subQuestions = paperDetailUnitExp.getQuestion().getSubQuestions();
                            if (subQuestions != null && subQuestions.size() > 0) {
                                for (int i = 0; i < subQuestions.size(); i++) {
                                    newPaperDetailUnits.add(new PaperDetailUnitExp(type, subQuestions.get(i)));
                                }
                            }
                        }
                    }
                    //设置小题
                    newPaperDetailExp.setPaperDetailUnits(newPaperDetailUnits);
                    //设置大题题号
                    newPaperDetailExp.setNumber(Integer.parseInt(type.getId() + ""));
                    newPaperDetailExp.setSortNumber(paperDetailExp.getSortNumber());
                    newObjs.add(newPaperDetailExp);
                }
            }
        }
        //刷一遍number
        int num = 0;
        for (QuesStructType type : types) {
            for (int i = 0; i < newObjs.size(); i++) {
                if (newObjs.get(i).getSortNumber().equals(type.getId())) {
                    List<PaperDetailUnitExp> exps = newObjs.get(i).getPaperDetailUnits();
                    for (int j = 0; j < exps.size(); j++) {
                        num++;
                        exps.get(j).setNumber(num);
                    }
                }
            }
        }
        return newObjs;
    }

    @Override
    public PaperExp getDownPaperExp(String paperId) throws Exception {
        Paper paper = Model.of(paperRepo.findById(paperId));
        // 创建paperDto
        PaperExp paperExp = BeanCopierUtil.copyProperties(paper, PaperExp.class);
        paperExp.setCourseName(paper.getCourse().getName());
        paperExp.setCourseNo(paper.getCourse().getCode());
        paperExp.setCourse(paper.getCourse());
        // 获取大题
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
        List<PaperDetailExp> paperDetailExps = new ArrayList<>();
        for (PaperDetail paperDetail : paperDetails) {
            PaperDetailExp paperDetailExp = new PaperDetailExp();
            BeanUtils.copyProperties(paperDetail, paperDetailExp);
            if (StringUtils.isBlank(paperDetailExp.getName())) {
                paperDetailExp.setName("默认大题");
            }
            paperDetailExps.add(paperDetailExp);
        }
        // 封装小题
        for (int i = 0; i < paperDetailExps.size(); i++) {
            List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetails.get(i));
            if (paperDetailUnits != null && paperDetailUnits.size() > 0) {
                List<PaperDetailUnitExp> paperDetailUnitExps = new ArrayList<>();
                for (PaperDetailUnit paperDetailUnit : paperDetailUnits) {
                    PaperDetailUnitExp paperDetailUnitExp = new PaperDetailUnitExp();
                    BeanUtils.copyProperties(paperDetailUnit, paperDetailUnitExp);
                    paperDetailUnitExps.add(paperDetailUnitExp);
                }
                paperDetailExps.get(i).setPaperDetailUnits(paperDetailUnitExps);
                paperDetailExps.get(i).setUnitCount(paperDetailUnitExps.size());
            } else {
                paperDetailExps.get(i).setUnitCount(0);
            }
        }
        paperExp.setPaperDetails(paperDetailExps);
        if (paperExp.getPaperDetails() == null || paperExp.getPaperDetails().size() == 0) {
            return paperExp;
        }
        int mainNum = 0;
        List<PaperDetailExp> paperDetailExpList = paperExp.getPaperDetails();
        for (PaperDetailExp paperDetail : paperDetailExpList) {
            // 大题序号
            paperDetail.setNumber(++mainNum);
            paperDetail.setCnNum(CommonUtils.toCHNum(paperDetail.getNumber()));
            if (paperDetail != null && paperDetail.getPaperDetailUnits() != null && paperDetail.getPaperDetailUnits().size() > 0) {
                int minNum = 1;
                for (PaperDetailUnitExp paperDetailUnit : paperDetail.getPaperDetailUnits()) {
                    Question question = paperDetailUnit.getQuestion();
                    question.setDifficultyDegree(question.getDifficultyDegree() * 10);
                    if (question.getPublicity() == null || question.getPublicity() == true) {
                        paperDetailUnit.setPublicity("公开");
                    } else {
                        paperDetailUnit.setPublicity("非公开");
                    }
                    if (StringUtils.isBlank(paperDetail.getQuesType())) {
                        paperDetail.setQuesType(question.getQuestionType().getName());
                    }
                    //设置选项
                    List<QuesOption> optionList = paperDetailUnit.getQuestion().getQuesOptions();
                    if (optionList != null && optionList.size() > 0) {
                        int index = 0;
                        for (QuesOption quesOption : optionList) {
                            quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(), getOptionNum(index)));
                            index++;
                        }
                    }
                    if (question.getQuestionType() != QuesStructType.NESTED_ANSWER_QUESTION) {
                        //给小题设置序号
                        question.setQuesBodyWord(setSubQuesNum(question.getQuesBodyWord(), minNum++));
                        question.setQuesAnswerWord(setAnswerNum(question.getQuesAnswerWord()));
                        if (paperDetail.getFirstScore() == null) {
                            if (question.getScore() == null) {
                                paperDetail.setFirstScore(0d);
                            } else {
                                paperDetail.setFirstScore(question.getScore());
                            }
                        }
                    } else {
                        List<Question> subQuesList = question.getSubQuestions();
                        // 套题小题设置序号
                        if (subQuesList != null && subQuesList.size() > 0) {
                            int index = 1;
                            for (Question subQues : subQuesList) {
                                subQues.setDifficultyDegree(subQues.getDifficultyDegree() * 10);
                                subQues.setQuesBodyWord(setSubQuesNum(subQues.getQuesBodyWord(), index++));
                                subQues.setQuesAnswerWord(setAnswerNum(subQues.getQuesAnswerWord()));
                                if (paperDetail.getFirstScore() == null) {
                                    if (subQues.getScore() == null) {
                                        paperDetail.setFirstScore(0d);
                                    } else {
                                        paperDetail.setFirstScore(subQues.getScore());
                                    }
                                }
                                List<QuesOption> subOptionList = subQues.getQuesOptions();
                                if (subOptionList != null && subOptionList.size() > 0) {
                                    int sub_index = 0;
                                    for (QuesOption quesOption : subOptionList) {
                                        quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(), getOptionNum(sub_index)));
                                        sub_index++;
                                    }
                                }
                            }
                        }
                    }
                    List<QuesProperty> quesProperties = question.getQuesProperties();
                    if (quesProperties != null && quesProperties.size() > 0) {
                        QuesProperty quesProperty = quesProperties.get(0);
                        paperDetailUnit.setFirstName(quesProperty.getFirstProperty().getName());
                        paperDetailUnit.setSecondName(quesProperty.getSecondProperty().getName());
                    }
                }
            }
        }
        return paperExp;
    }

    /**
     * 设置题号
     *
     * @param quesBodyWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setSubQuesNum(String quesBodyWordMl, int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body) XmlUtils.unmarshalString(tmpStr, Context.jc, Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for (Object pObj : pList) {
            if (index > 0) {
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num + ". ");
            run.getContent().add(text);
            pContent.add(0, run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for (Object pObj : pList) {
            if (pObj instanceof P) {
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    /**
     * 将数字1,2,3,4转化成A,B,C,D
     *
     * @param number
     * @return
     */
    public String getOptionNum(int number) {
        char optionNum = (char) (65 + number);
        return String.valueOf(optionNum);
    }

    /**
     * 设置选项号
     *
     * @param optionWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setOptionNum(String optionWordMl, String num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + optionWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body) XmlUtils.unmarshalString(tmpStr, Context.jc, Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for (Object pObj : pList) {
            if (index > 0) {
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num + ". ");
            run.getContent().add(text);
            pContent.add(0, run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for (Object pObj : pList) {
            if (pObj instanceof P) {
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    public String setAnswerNum(String quesBodyWordMl) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body) XmlUtils.unmarshalString(tmpStr, Context.jc, Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for (Object pObj : pList) {
            if (index > 0) {
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue("[答案]:");
            run.getContent().add(text);
            pContent.add(0, run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for (Object pObj : pList) {
            if (pObj instanceof P) {
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    @Override
    public int getQuestionTypeNumbers(String paperId, Integer publicityType, Integer difficultyType) {
        if (publicityType == null || difficultyType == null) {
            return 0;
        }
        Boolean publicity = null;
        String difficulty = null;
        if (publicityType == 1) {
            publicity = true;
        } else {
            publicity = false;
        }
        if (difficultyType == 1) {
            difficulty = "易";
        } else if (difficultyType == 2) {
            difficulty = "中";
        } else {
            difficulty = "难";
        }
        String needType = publicity + "-" + difficulty;
        Paper paper = Model.of(paperRepo.findById(paperId));
        List<PaperDetailUnit> units = paperDetailUnitRepo.findByPaperOrderByNumber(paper);
        if (units != null && units.size() > 0) {
            int total = 0;
            for (PaperDetailUnit unit : units) {
                if (unit.getQuestion().getPublicity() == null) {
                    unit.getQuestion().setPublicity(true);
                }
                if (unit.getQuestion().getDifficulty() == null) {
                    unit.getQuestion().setDifficulty("中");
                }
                String factType = unit.getQuestion().getPublicity() + "-" + unit.getQuestion().getDifficulty();
                if (needType.equals(factType)) {
                    total += 1;
                }
            }
            return total;
        }
        return 0;
    }

    @Override
    public double getQuestionTypeScore(String paperId, Integer publicityType, Integer difficultyType) {
        if (publicityType == null || difficultyType == null) {
            return 0;
        }
        Boolean publicity = null;
        String difficulty = null;
        if (publicityType == 1) {
            publicity = true;
        } else {
            publicity = false;
        }
        if (difficultyType == 1) {
            difficulty = "易";
        } else if (difficultyType == 2) {
            difficulty = "中";
        } else {
            difficulty = "难";
        }
        String needType = publicity + "-" + difficulty;
        Paper paper = Model.of(paperRepo.findById(paperId));
        List<PaperDetailUnit> units = paperDetailUnitRepo.findByPaperOrderByNumber(paper);
        if (units != null && units.size() > 0) {
            double total = 0;
            BigDecimal b1 = BigDecimal.valueOf(total);
            for (PaperDetailUnit unit : units) {
                if (unit.getQuestion().getPublicity() == null) {
                    unit.getQuestion().setPublicity(true);
                }
                if (unit.getQuestion().getDifficulty() == null) {
                    unit.getQuestion().setDifficulty("中");
                }
                String factType = unit.getQuestion().getPublicity() + "-" + unit.getQuestion().getDifficulty();
                if (needType.equals(factType)) {
                    BigDecimal b2 = BigDecimal.valueOf(unit.getScore());
                    b1 = b1.add(b2);
                }
            }
            return b1.doubleValue();
        }
        return 0;
    }

    private void clearPaperCache(String paperId) {
        //清理与当前试卷相关的缓存
        final String patternKey = CACHE_KEY_PAPER + "*" + paperId;
        Set<String> keys = redisTemplate.keys(patternKey);
        if (CollectionUtils.isNotEmpty(keys)) {
            redisTemplate.delete(keys);
        }
    }

    private void clearQuestionCache(String questionId) {
        //清理与当前试题相关的缓存
        final String patternKey = CACHE_KEY_QUESTION + "*" + questionId;
        Set<String> keys = redisTemplate.keys(patternKey);
        if (CollectionUtils.isNotEmpty(keys)) {
            redisTemplate.delete(keys);
        }
    }

}