package cn.com.qmth.examcloud.core.questions.starter;

import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.autoconfigure.web.servlet.MultipartAutoConfiguration;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.eureka.EnableEurekaClient;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

import cn.com.qmth.examcloud.web.bootstrap.AppBootstrap;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import cn.com.qmth.examcloud.web.upyun.UpyunSiteManager;

@SpringBootApplication
@Configuration
@EnableJpaAuditing
@EnableTransactionManagement
@EnableEurekaClient
@EnableDiscoveryClient
@ComponentScan(basePackages = { "cn.com.qmth" })
@EntityScan(basePackages = { "cn.com.qmth" })
@EnableJpaRepositories(basePackages = { "cn.com.qmth" })
@EnableMongoRepositories(basePackages = { "cn.com.qmth" })
@EnableAutoConfiguration(exclude = { MultipartAutoConfiguration.class })
public class CoreQuestionApp {

    static {
        String runtimeLevel = System.getProperty("log.commonLevel");
        if (null == runtimeLevel) {
            System.setProperty("log.commonLevel", "DEBUG");
        }
        System.setProperty("hibernate.dialect.storage_engine", "innodb");
    }

    public static void main(String[] args) {
        AppBootstrap.run(CoreQuestionApp.class, args);
        UpyunSiteManager.init();
        test();
    }

    private static void test() {
        Tester tester = SpringContextHolder.getBean(Tester.class);
        tester.test();
    }

    @Bean(name = "multipartResolver")
    public MultipartResolver multipartResolver() {
        CommonsMultipartResolver resolver = new CommonsMultipartResolver();
        resolver.setDefaultEncoding("UTF-8");
        resolver.setResolveLazily(true);
        resolver.setMaxInMemorySize(40960);
        resolver.setMaxUploadSize(200L * 1024L * 1024L);
        return resolver;
    }

}